import objectdraw.*;
import java.awt.*;
import java.util.ArrayList;

/* $Id: DragAllRoadsForeach.java 2346 2014-04-15 13:17:42Z terescoj $ */

/**
 * Example DragAllRoadsForeach: draw "road segments" at mouse
 * press points, but drag any existing segment instead
 * if it happens to contain the mouse press point, using 
 * a for each loop to traverse.
 *
 * @author Jim Teresco
 * The College of Saint Rose, CSC 252, Spring 2014
 *
 */

public class DragAllRoadsForeach extends WindowController {
    
    // the road segments in our drawing
    private ArrayList<RoadSegment> segments;
    
    // the road segment currently being dragged, if any
    private RoadSegment selectedSegment;
    
    // last mouse point and boolean flag to support dragging
    private boolean dragging;
    private Location lastMouse;
    
    /**
     * Set up: just initialize the array of segments
     * and set other instance variables.
     */
    public void begin() {
    
        segments = new ArrayList<RoadSegment>();
        selectedSegment = null;
        dragging = false;
        lastMouse = null;
    }
    
    /** Draw a road segment at the mouse press point
     *  unless the most recent road drawn was at that
     *  point, in which case we begin a drag operation.
     * 
     *  @param point the Location of the mouse press
     */
    public void onMousePress(Location point) {
    
        // see if we've pressed the mouse on an existing road segment
        for (RoadSegment segment : segments) {
            
            if (segment.contains(point)) {
                selectedSegment = segment;
                dragging = true;
                lastMouse = point;
            }
        }
        
        // if dragging is still false, we create a new segment
        if (!dragging) {
            
            // create the new segment and insert it into the list.
            segments.add(new RoadSegment(point, canvas));
            dragging = false;
        }
    }
    
    /** Perform dragging operation if we are dragging anything
     * 
     *  @param point the Location of the mouse press
     */
    public void onMouseDrag(Location point) {
     
        if (dragging) {
            selectedSegment.move(point.getX() - lastMouse.getX(),
                                 point.getY() - lastMouse.getY());
            lastMouse = point;
        }
    }
    
    /** Complete dragging operation if we are dragging anything
     * 
     *  @param point the Location of the mouse press
     */
    public void onMouseRelease(Location point) {
     
        if (dragging) {
            selectedSegment.move(point.getX() - lastMouse.getX(),
                                 point.getY() - lastMouse.getY());
            dragging = false;      
        }
    }  
    
    public void onMouseExit(Location point) {
        
        if (dragging) {
            selectedSegment.removeFromCanvas();
            segments.remove(selectedSegment);
            dragging = false;
        }
    }
 }
